@@ -1,5 +1,5 @@
 package Mojo::Weixin;
-our $VERSION = '1.2.2';
+our $VERSION = '1.2.3';
 use Mojo::Weixin::Base 'Mojo::EventEmitter';
 use Mojo::IOLoop;
 use Mojo::Weixin::Log;
@@ -17,6 +17,8 @@ has log_path            => undef;
 has log_encoding        => undef;      #utf8|gbk|...
 has log_head            => undef;
 has log_unicode         => 0;
+has log_console         => 1;
+has download_media      => 1;
 
 has account             => sub{ $ENV{MOJO_WEIXIN_ACCUNT} || 'default'};
 has start_time          => time;
@@ -25,12 +27,17 @@ has media_dir           => sub {$_[0]->tmpdir};
 has cookie_path         => sub {File::Spec->catfile($_[0]->tmpdir,join('','mojo_weixin_cookie_',$_[0]->account || 'default','.dat'))};
 has qrcode_path         => sub {File::Spec->catfile($_[0]->tmpdir,join('','mojo_weixin_qrcode_',$_[0]->account || 'default','.jpg'))};
 has pid_path            => sub {File::Spec->catfile($_[0]->tmpdir,join('','mojo_weixin_pid_',$_[0]->account || 'default','.pid'))};
+has state_path          => sub {File::Spec->catfile($_[0]->tmpdir,join('','mojo_weixin_state_',$_[0]->account || 'default','.json'))};
 has ioloop              => sub {Mojo::IOLoop->singleton};
 has keep_cookie         => 1;
 has fix_media_loop      => 1;
 has synccheck_interval  => 1;
+has synccheck_delay     => 10;
+has _synccheck_interval => sub{ $_[0]->synccheck_interval};
+has sync_interval       => 0;
 has emoji_to_text       => 1;
 has stop_with_mobile    => 0;
+has http_max_message_size  => undef; #16777216;
 
 has user    => sub {+{}};
 has friend  => sub {[]};
@@ -46,6 +53,7 @@ has log     => sub{
         path        =>  $_[0]->log_path,
         level       =>  $_[0]->log_level,
         unicode_support => $_[0]->log_unicode,
+        console_output  => $_[0]->log_console,
         format      =>  sub{
             my ($time, $level, @lines) = @_;
             my $title = "";
@@ -70,20 +78,25 @@ has log     => sub{
 has is_ready                => 0;
 has is_stop                 => 0;
 has ua_retry_times          => 5;
+has ua_connect_timeout      => 10;
+has ua_request_timeout      => 35;
+has ua_inactivity_timeout   => 35;
 has is_first_login          => -1;
 has login_state             => 'init';
 has qrcode_count            => 0;
 has qrcode_count_max        => 10;
 has ua                      => sub {
     #local $ENV{MOJO_USERAGENT_DEBUG} = $_[0]->ua_debug;
+    local $ENV{MOJO_MAX_MESSAGE_SIZE} = $_[0]->http_max_message_size if defined $_[0]->http_max_message_size;
     require Mojo::UserAgent;
     require Mojo::UserAgent::Proxy;
     require Storable if $_[0]->keep_cookie;
     Mojo::UserAgent->new(
         proxy              => sub{ my $proxy = Mojo::UserAgent::Proxy->new;$proxy->detect;$proxy}->(),
         max_redirects      => 7,
-        request_timeout    => 120,
-        inactivity_timeout => 120,
+        connect_timeout    => $_[0]->ua_connect_timeout,
+        request_timeout    => $_[0]->ua_request_timeout,
+        inactivity_timeout => $_[0]->ua_inactivity_timeout,
         transactor => Mojo::UserAgent::Transactor->new( 
             name =>  'Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2062'
         ),
@@ -106,6 +119,19 @@ has _synccheck_error_count => 0;
 has _synccheck_connection_id => undef;
 
 sub deviceid { return "e" . substr(rand() . ("0" x 15),2,15);}
+sub state {
+    my $self = shift;
+    $self->{state} = 'init' if not defined $self->{state};
+    if(@_ == 0){#get
+        return $self->{state};
+    } 
+    elsif($_[0] and $_[0] ne $self->{state}){#set
+        my($old,$new) = ($self->{state},$_[0]);
+        $self->{state} = $new;
+        $self->emit(state_change=>$old,$new);
+    }
+    $self;
+}
 sub on {
     my $self = shift;
     my @return;
@@ -175,13 +201,19 @@ sub new {
         $self->error(Carp::longmess($err));
     });
     $self->check_pid();
+    $self->save_state();
     $SIG{CHLD} = 'IGNORE';
     $SIG{INT} = $SIG{KILL} = $SIG{TERM} = $SIG{HUP} = sub{
-        $self->info("正在停止客户端...");
+        return if $^O ne 'MSWin32' and $_[0] eq 'INT' and !-t;
+        $self->info("捕获到停止信号[$_[0]]，准备停止...");
         $self->clean_qrcode();
         $self->clean_pid();
         $self->stop();
     };
+    $self->on(state_change=>sub{
+        my $self = shift;
+        $self->save_state();
+    });
     $self->on(qrcode_expire=>sub{
         my($self) = @_;
         my $count = $self->qrcode_count;
