@@ -1,13 +1,14 @@
 package Mojo::Weixin::Friend;
 use Mojo::Weixin::Base 'Mojo::Weixin::Model::Base';
 use List::Util qw(first);
-has name => '昵称未知';
+has name => '';
 has [qw( 
     account
     province
     city
     sex
     id
+    uid
     signature
     display
     markname
@@ -22,12 +23,15 @@ sub new {
     my $self = shift;
     $self = $self->Mojo::Weixin::Base::new(@_);
     $self->client->emoji_convert(\$self->{name},$self->client->emoji_to_text);
+    $self->client->emoji_convert(\$self->{display},$self->client->emoji_to_text);
+    $self->client->emoji_convert(\$self->{markname},$self->client->emoji_to_text);
     if(exists $special_id{$self->id}){
         $self->category("系统帐号");
     }
     elsif(defined $self->_verifyflag and $self->_verifyflag & 8){
         $self->category("公众号");
     }
+    $self->uid("") if not $self->uid;
     $self;
 }
 sub get_avatar{
@@ -37,7 +41,7 @@ sub get_avatar{
 
 sub displayname{
     my $self = shift;
-    return $self->display || $self->markname || $self->name;
+    return $self->display || $self->markname || $self->name || '昵称未知';
 }
 
 sub update{
@@ -46,6 +50,8 @@ sub update{
     for(grep {substr($_,0,1) ne "_"} keys %$hash){
         if(exists $hash->{$_}){
             $self->client->emoji_convert(\$hash->{$_},$self->client->emoji_to_text) if $_ eq "name";
+            $self->client->emoji_convert(\$hash->{$_},$self->client->emoji_to_text) if $_ eq "display";
+            $self->client->emoji_convert(\$hash->{$_},$self->client->emoji_to_text) if $_ eq "markname";
             if(defined $hash->{$_} and defined $self->{$_}){
                 if($hash->{$_} ne $self->{$_}){
                     my $old_property = $self->{$_};
@@ -76,6 +82,6 @@ sub send_media{
 }
 sub set_markname {
     my $self = shift;
-    $self->client->set_friend_markname($self,@_);
+    $self->client->set_markname($self,@_);
 }
 1;
