@@ -3,6 +3,8 @@ use Mojo::Weixin::Base 'Mojo::Weixin::Model::Base';
 use Mojo::Weixin::Group::Member;
 
 has 'id';
+has 'uid';
+has 'owner_uid';
 has name => '';
 has member => sub{[]};
 has _avatar => '';
@@ -21,6 +23,7 @@ sub new {
     my $class = shift;
     my $self;
     bless $self=@_ ? @_ > 1 ? {@_} : {%{$_[0]}} : {}, ref $class || $class;
+    $self->uid("") if not $self->uid;
     if(exists $self->{member} and ref $self->{member} eq "ARRAY"){
         for( @{ $self->{member} } ){
             $_ = Mojo::Weixin::Group::Member->new($_) if ref $_ ne "Mojo::Weixin::Group::Member";
@@ -151,5 +154,9 @@ sub kick_group_member{
     my $self = shift;
     $self->client->kick_group_member($self,@_);
 }
+sub sticky_group{
+    my $self = shift;
+    $self->client->sticky_group($self,@_);
+}
 
 1;
