@@ -8,7 +8,8 @@ use Mojo::Weixin::Model::Remote::_webwxbatchgetcontact;
 use Mojo::Weixin::Model::Remote::_webwxstatusnotify;
 use Mojo::Weixin::Model::Remote::_webwxcreatechatroom;
 use Mojo::Weixin::Model::Remote::_webwxupdatechatroom;
-use Mojo::Weixin::Model::Remote::_webwxoplog;
+use Mojo::Weixin::Model::Remote::_webwxoplog_markname;
+use Mojo::Weixin::Model::Remote::_webwxoplog_sticky;
 use Mojo::Weixin::Model::Remote::_webwxverifyuser;
 use Mojo::Weixin::Model::Remote::_webwxgetheadimg;
 use Mojo::Weixin::User;
@@ -17,6 +18,7 @@ use Mojo::Weixin::Const;
 
 sub model_init{
     my $self = shift;
+    $self->state('updating');
     $self->info("获取联系人信息...");
     my $initinfo = $self->_webwxinit();
     if(not defined $initinfo){
@@ -28,6 +30,7 @@ sub model_init{
         $self->info("更新个人信息成功");
         $self->user(Mojo::Weixin::User->new($user));
         $self->_webwxstatusnotify($self->user->id,3);
+        $self->emit(update_user=>$self->user);
     }
     my $contactinfo = $self->_webwxgetcontact();
     if(not defined $contactinfo){
@@ -38,6 +41,7 @@ sub model_init{
     if(ref $friends eq "ARRAY" and @$friends>0){
         $self->friend([ map {Mojo::Weixin::Friend->new($_)} grep {$_->{id} ne $user->{id}} @$friends ]);
         $self->info("更新好友信息成功");
+        $self->emit(update_friend=>$self->friend);
     }
 
     my %groups_id;
@@ -58,6 +62,7 @@ sub model_init{
             if(ref $groups eq "ARRAY" and @$groups >0){
                 $self->group([ map { Mojo::Weixin::Group->new($_) } @$groups ]);
                 $self->info("更新群组[ @{[$_->displayname]} ]信息成功") for $self->groups;
+                $self->emit(update_group=>$self->group);
             }
         }
         else{
@@ -167,7 +172,7 @@ sub remove_group{
 sub is_group{
     my $self = shift;
     my $gid = shift;
-    return index($gid,'@@')==0?1:0;
+    return $gid=~/^\@\@|\@chatroom$/ ? 1 : 0;
 }
 sub code2sex{
     my $c = shift;
@@ -205,27 +210,48 @@ sub groups{
     return @{$self->group};
 }
 
-sub set_friend_markname {
+sub set_markname {
     my $self = shift;
-    my $friend = shift;
+    my $object = shift;
     my $markname = shift;
-    if(ref $friend ne "Mojo::Weixin::Friend"){
+    if(ref $object ne "Mojo::Weixin::Friend" and ref $object ne "Mojo::Weixin::Group::Member"){
         $self->die("无效的对象数据类型");
         return;
     }
-    my $displayname = $friend->displayname;
-    my $ret = $self->_webwxoplog($friend->id,$markname);
+    $self->warn("设置群成员备注的功能当前可能已被官方屏蔽") if ref $object ne "Mojo::Weixin::Group::Member";
+    my $displayname = $object->displayname;
+    my $ret = $self->_webwxoplog_markname($object->id,$markname);
     if($ret){
-        $self->info("设置好友 $displayname 备注[ $markname ]成功");
+        $self->info("设置对象 $displayname 备注[ $markname ]成功");
         return 1;
     }
     else{
-        $self->info("设置好友 $displayname 备注[ $markname ]失败");
+        $self->info("设置对象 $displayname 备注[ $markname ]失败");
         return 0;
     }
 
 }
 
+sub sticky_group{
+    my $self = shift;
+    my $object = shift;
+    my $op = shift;
+    if(ref $object ne "Mojo::Weixin::Group"){
+        $self->die("无效的对象数据类型");
+        return;
+    }
+    my $ret = $self->_webwxoplog_sticky($object->id,$op // 1);
+    my $displayname = $object->displayname;
+    if($ret){
+        $op?$self->info("设置群组[ $displayname ]置顶成功"): $self->info("取消群组[ $displayname ]置顶成功");
+        return 1;
+    }
+    else{
+        $op?$self->info("设置群组[ $displayname ]置顶失败"): $self->info("取消群组[ $displayname ]置顶失败");
+        return 0;
+    }
+
+}
 sub create_group {
     my $self = shift;
     my $friends;
