@@ -4,31 +4,42 @@ use strict;
 use Encode;
 use POSIX qw();
 use Mojo::Util qw();
+use List::Util qw(first);
 use Mojo::Weixin::Server;
 my $server;
 sub call{
     my $client = shift;
     my $data   =  shift;
     my $post_api = $data->{post_api} if ref $data eq "HASH";
+    my $poll_api = $data->{poll_api} if ref $data eq "HASH";
     $data->{post_media_data} = 1 if not defined $data->{post_media_data};
+    $data->{post_event_list} = [qw(login stop state_change input_qrcode new_group new_friend new_group_member lose_group lose_friend lose_group_member)] 
+        if ref $data->{post_event_list} ne 'ARRAY'; 
 
+    if(defined $poll_api){
+        $client->on('_mojo_weixin_plugin_openwx_poll_over' => sub{
+            $client->http_get($poll_api,sub{
+                $client->timer($data->{poll_interval} || 5,sub {$client->emit('_mojo_weixin_plugin_openwx_poll_over');});
+            });
+        });
+        $client->emit('_mojo_weixin_plugin_openwx_poll_over');
+    }
     if(defined $post_api){
         $client->on(all_event => sub{
             my($client,$event,@args) =@_;
-            if($event eq  'login' or $event eq 'stop'){
+            return if not first {$event eq $_} @{ $data->{post_event_list} };
+            if($event eq  'login' or $event eq 'stop' or $event eq 'state_change'){
                 my $post_json = {};
                 $post_json->{post_type} = "event";
                 $post_json->{event} = $event;
                 $post_json->{params} = [@args];
-                $client->http_post($post_api,json=>$post_json,sub{
-                    my($data,$ua,$tx) = @_;
-                    if($tx->success){
-                        $client->debug("插件[".__PACKAGE__ ."]事件[".$event . "](@args)上报成功");
-                    }
-                    else{
-                        $client->warn("插件[".__PACKAGE__ . "]事件[".$event."](@args)上报失败:" . encode("utf8",$tx->error->{message}));
-                    } 
-                });
+                my($data,$ua,$tx) = $client->http_post($post_api,{ua_connect_timeout=>5,ua_request_timeout=>5,ua_inactivity_timeout=>5,ua_retry_times=>2},json=>$post_json);
+                if($tx->success){
+                    $client->debug("插件[".__PACKAGE__ ."]事件[".$event . "](@args)上报成功");
+                }
+                else{
+                    $client->warn("插件[".__PACKAGE__ . "]事件[".$event."](@args)上报失败:" . encode("utf8",$tx->error->{message}));
+                } 
             }
             elsif($event eq 'input_qrcode'){
                 my ($qrcode_path,$qrcode_data) = @args;
@@ -171,6 +182,10 @@ sub call{
         return if not defined $jsoncallback;
         $$output = "$jsoncallback($$output)";
     });
+    helper safe_render =>sub {
+        my $c = shift;
+        $c->render(@_) if (defined $c->tx and !$c->tx->is_finished); 
+    };
     under sub {
         my $c = shift;
         if(ref $data eq "HASH" and ref $data->{auth} eq "CODE"){
@@ -181,24 +196,24 @@ sub call{
                 $ret = $data->{auth}->($hash,$c);
             };
             $client->warn("插件[Mojo::Weixin::Plugin::Openwx]认证回调执行错误: $@") if $@;
-            $c->render(text=>"auth failure",status=>403) if not $ret;
+            $c->safe_render(text=>"auth failure",status=>403) if not $ret;
             return $ret;
         }
         else{return 1} 
     };
-    get '/openwx/get_user_info'     => sub {$_[0]->render(json=>$client->user->to_json_hash());};
-    get '/openwx/get_friend_info'   => sub {$_[0]->render(json=>[map {$_->to_json_hash()} @{$client->friend}]); };
-    get '/openwx/get_group_info'    => sub {$_[0]->render(json=>[map {$_->to_json_hash()} @{$client->group}]); };
+    get '/openwx/get_user_info'     => sub {$_[0]->safe_render(json=>$client->user->to_json_hash());};
+    get '/openwx/get_friend_info'   => sub {$_[0]->safe_render(json=>[map {$_->to_json_hash()} @{$client->friend}]); };
+    get '/openwx/get_group_info'    => sub {$_[0]->safe_render(json=>[map {$_->to_json_hash()} @{$client->group}]); };
     any [qw(GET POST)] => '/openwx/search_friend' => sub{
         my $c = shift;
         $c->req->params->remove('client');
         my @params = map {defined $_?Encode::encode("utf8",$_):$_} @{$c->req->params->pairs};
         my @objects = $client->search_friend(@params);
         if(@objects){
-            $c->render(json=>[map {$_->to_json_hash()} @objects]);
+            $c->safe_render(json=>[map {$_->to_json_hash()} @objects]);
         }
         else{
-            $c->render(json=>{code=>100,status=>"object not found"});
+            $c->safe_render(json=>{code=>100,status=>"object not found"});
         }
     };
     any [qw(GET POST)] => '/openwx/search_group' => sub{
@@ -207,10 +222,10 @@ sub call{
         my @params = map {defined $_?Encode::encode("utf8",$_):$_} @{$c->req->params->pairs};
         my @objects = $client->search_group(@params);
         if(@objects){
-            $c->render(json=>[map {$_->to_json_hash()} @objects]);
+            $c->safe_render(json=>[map {$_->to_json_hash()} @objects]);
         }
         else{
-            $c->render(json=>{code=>100,status=>"object not found"});
+            $c->safe_render(json=>{code=>100,status=>"object not found"});
         }
     };
     any [qw(GET POST)] => '/openwx/send_friend_message'         => sub{
@@ -247,7 +262,7 @@ sub call{
                     );
                 }
             }
-            $c->render(json=>{msg_id=>0,code=>0,status=>'request already executed'});
+            $c->safe_render(json=>{msg_id=>0,code=>0,status=>'request already executed'});
             return;
         }
         my $object = $client->search_friend(id=>$id,account=>$account,displayname=>$displayname,markname=>$markname);
@@ -257,7 +272,7 @@ sub call{
                 my $msg= $_[1];
                 $msg->cb(sub{
                     my($client,$msg,$status)=@_;
-                    $c->render(json=>{msg_id=>$msg->id,code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
+                    $c->safe_render(json=>{msg_id=>$msg->id,code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
                 });
                 $msg->from("api");
             }) if defined $content;
@@ -276,13 +291,13 @@ sub call{
                     my $msg= $_[1];
                     $msg->cb(sub{
                         my($client,$msg,$status)=@_;
-                        $c->render(json=>{msg_id=>$msg->id,media_id=>join(":",$msg->media_id,$msg->media_code),code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
+                        $c->safe_render(json=>{msg_id=>$msg->id,media_id=>join(":",$msg->media_id,$msg->media_code),code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
                     });
                     $msg->from("api");
                 });
             }
         }
-        else{$c->render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
+        else{$c->safe_render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
     };
     any [qw(GET POST)] => '/openwx/send_group_message'         => sub{
         my $c = shift;
@@ -307,7 +322,7 @@ sub call{
                 my $msg= $_[1];
                 $msg->cb(sub{
                     my($client,$msg,$status)=@_;
-                    $c->render(json=>{msg_id=>$msg->id,code=>$status->code,status=>decode("utf8",$status->msg)});
+                    $c->safe_render(json=>{msg_id=>$msg->id,code=>$status->code,status=>decode("utf8",$status->msg)});
                 });
                 $msg->from("api");
             }) if defined $content;
@@ -326,13 +341,13 @@ sub call{
                     my $msg= $_[1];
                     $msg->cb(sub{
                         my($client,$msg,$status)=@_;
-                        $c->render(json=>{msg_id=>$msg->id,media_id=>join(":",$msg->media_id,$msg->media_code),code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
+                        $c->safe_render(json=>{msg_id=>$msg->id,media_id=>join(":",$msg->media_id,$msg->media_code),code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
                     });
                     $msg->from("api");
                 });
             }
         }
-        else{$c->render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
+        else{$c->safe_render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
     }; 
     any [qw(GET POST)] => '/openwx/consult'         => sub{
         my $c = shift;
@@ -360,12 +375,12 @@ sub call{
                     my ($timer,$cb);
                     $timer = Mojo::IOLoop->timer($timeout || 30,sub{
                         $client->unsubscribe(receive_message=>$cb);
-                        $c->render(json=>{msg_id=>$msg->id,code=>$status->code,status=>Encode::decode("utf8",$status->msg),reply_status=>"reply timeout",reply=>undef});
+                        $c->safe_render(json=>{msg_id=>$msg->id,code=>$status->code,status=>Encode::decode("utf8",$status->msg),reply_status=>"reply timeout",reply=>undef});
                     });
                     $cb = $client->once(receive_message=>sub{
                         my($client,$msg) = @_;
                         Mojo::IOLoop->remove($timer);
-                        $c->render(json=>{reply=>Encode::decode("utf8",$msg->content),msg_id=>$msg->id,code=>$status->code,status=>Encode::decode("utf8",$status->msg)}); 
+                        $c->safe_render(json=>{reply=>Encode::decode("utf8",$msg->content),msg_id=>$msg->id,code=>$status->code,status=>Encode::decode("utf8",$status->msg)}); 
                     });
                 });
                 $msg->from("api");
@@ -385,13 +400,13 @@ sub call{
                     my $msg= $_[1];
                     $msg->cb(sub{
                         my($client,$msg,$status)=@_;
-                        $c->render(json=>{msg_id=>$msg->id,media_id=>join(":",$msg->media_id,$msg->media_code),code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
+                        $c->safe_render(json=>{msg_id=>$msg->id,media_id=>join(":",$msg->media_id,$msg->media_code),code=>$status->code,status=>Encode::decode("utf8",$status->msg)});
                     });
                     $msg->from("api");
                 });
             }
         }
-        else{$c->render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
+        else{$c->safe_render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
     };
     any [qw(GET POST)] => '/openwx/create_group' => sub{
         my $c = shift;
@@ -402,27 +417,27 @@ sub call{
             for(@id){
                 my $friend = $client->search_friend(id=>$_);
                 if(not defined $friend){
-                    $c->render(json=>{code=>100,status=>"friend id $_ not found"});
+                    $c->safe_render(json=>{code=>100,status=>"friend id $_ not found"});
                     return;
                 }
                 push @friends,$friend;
             } 
             my $group = $client->create_group(\@friends,$displayname);
             if(defined $group){
-                $c->render(json=>{code=>0,group_id=>$group->id,status=>"success"});
+                $c->safe_render(json=>{code=>0,group_id=>$group->id,status=>"success"});
             }
             else{
-                $c->render(json=>{code=>201,id=>undef,status=>"failure"});
+                $c->safe_render(json=>{code=>201,id=>undef,status=>"failure"});
             }
         }
-        else{$c->render(json=>{code=>200,status=>"friend id empty"});}
+        else{$c->safe_render(json=>{code=>200,status=>"friend id empty"});}
     };
     any [qw(GET POST)] => '/openwx/invite_friend' => sub{
         my $c = shift;
         my($id,$displayname,$friends)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("displayname"),$c->param("friend"));
         my $object = $client->search_group(id=>$id,displayname=>$displayname,);
         if(not defined $object){
-            $c->render(json=>{code=>100,status=>"object not found"});
+            $c->safe_render(json=>{code=>100,status=>"object not found"});
             return;    
         }
         my @id = split /,/,$friends;
@@ -431,19 +446,19 @@ sub call{
             for(@id){
                 my $friend = $client->search_friend(id=>$_);
                 if(not defined $friend){
-                    $c->render(json=>{code=>100,status=>"friend id $_ not found"});
+                    $c->safe_render(json=>{code=>100,status=>"friend id $_ not found"});
                     return;
                 }
                 push @friends,$friend;
             }
             if($object->invite_friend(@friends)){
-                $c->render(json=>{code=>0,status=>"success"});   
+                $c->safe_render(json=>{code=>0,status=>"success"});   
             }
             else{
-                $c->render(json=>{code=>201,status=>"failure"});
+                $c->safe_render(json=>{code=>201,status=>"failure"});
             }
         }
-        else{$c->render(json=>{code=>200,status=>"friend id empty"});}
+        else{$c->safe_render(json=>{code=>200,status=>"friend id empty"});}
         
     };
     any [qw(GET POST)] => '/openwx/kick_group_member' => sub{
@@ -451,7 +466,7 @@ sub call{
         my($id,$displayname,$members)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("displayname"),$c->param("member"));
         my $object = $client->search_group(id=>$id,displayname=>$displayname,);
         if(not defined $object){
-            $c->render(json=>{code=>100,status=>"object not found"});
+            $c->safe_render(json=>{code=>100,status=>"object not found"});
             return;
         }
         my @id = split /,/,$members;
@@ -460,19 +475,19 @@ sub call{
             for(@id){
                 my $member = $object->search_group_member(id=>$_);
                 if(not defined $member){
-                    $c->render(json=>{code=>100,status=>"member id $_ not found"});
+                    $c->safe_render(json=>{code=>100,status=>"member id $_ not found"});
                     return;
                 }
                 push @members,$member;
             }
             if($object->kick_group_member(@members)){
-                $c->render(json=>{code=>0,status=>"success"});
+                $c->safe_render(json=>{code=>0,status=>"success"});
             }
             else{
-                $c->render(json=>{code=>201,status=>"failure"});
+                $c->safe_render(json=>{code=>201,status=>"failure"});
             }
         }
-        else{$c->render(json=>{code=>200,status=>"member id empty"});}
+        else{$c->safe_render(json=>{code=>200,status=>"member id empty"});}
         
     };
     any [qw(GET POST)] => '/openwx/set_group_displayname' => sub{
@@ -481,28 +496,70 @@ sub call{
         my $object = $client->search_group(id=>$id,displayname=>$displayname);
         if(defined $object){
             if($object->set_displayname($new_displayname)){
-                $c->render(json=>{code=>0,status=>"success"});
+                $c->safe_render(json=>{code=>0,status=>"success"});
             }
             else{
-                $c->render(json=>{code=>201,status=>"failure"});
+                $c->safe_render(json=>{code=>201,status=>"failure"});
             }
             
         }
-        else{$c->render(json=>{code=>100,status=>"object not found"});}
+        else{$c->safe_render(json=>{code=>100,status=>"object not found"});}
     };
-    any [qw(GET POST)] => '/openwx/set_friend_markname' => sub{
+    any [qw(GET POST)] => '/openwx/sticky_group' => sub{
         my $c = shift;
-        my($id,$account,$displayname,$markname,$new_markname)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("account"),$c->param("displayname"),$c->param("markname"),$c->param("new_markname"));
+        my($id,$displayname,$op)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("displayname"),$c->param("op"));
+        my $object = $client->search_group(id=>$id,displayname=>$displayname);
+        if(defined $object){
+            if($object->sticky_group($op)){
+                $c->safe_render(json=>{code=>0,status=>"success"});
+            }
+            else{
+                $c->safe_render(json=>{code=>201,status=>"failure"});
+            }
+
+        }
+        else{$c->safe_render(json=>{code=>100,status=>"object not found"});}
+    };
+    any [qw(GET POST)] => '/openwx/set_friend_markname' => sub {
+        my $c = shift;
+        my($id,$markname,$new_markname,$account,$displayname)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("markname"),$c->param("new_markname"),$c->param("account"),$c->param("displayname"));
         my $object = $client->search_friend(id=>$id,account=>$account,displayname=>$displayname,markname=>$markname);
         if(defined $object){
             if($object->set_markname($new_markname)){
-                $c->render(json=>{code=>0,status=>"success"});
+                $c->safe_render(json=>{code=>0,status=>"success"});
             }
             else{
-                $c->render(json=>{code=>201,status=>"failure"});
+                $c->safe_render(json=>{code=>201,status=>"failure"});
             }
         }
-        else{$c->render(json=>{code=>100,status=>"object not found"});}
+        else{$c->safe_render(json=>{code=>100,status=>"object not found"});}
+    };
+    any [qw(GET POST)] => '/openwx/set_markname' => sub{
+        my $c = shift;
+        my($id,$markname,$new_markname,$group_id,$account,$displayname)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("markname"),$c->param("new_markname"),$c->param("group_id"),$c->param("account"),$c->param("displayname"));
+        my $object;
+        if(defined $group_id){
+            my $group = $client->search_group(id=>$group_id);
+            if(not defined $group){
+                $c->safe_render(json=>{code=>100,status=>"group not found"});
+                return;
+            }
+            else{
+                $object = $group->search_group_member(id=>$id);
+            }
+        }
+        else{
+            $object = $client->search_friend(id=>$id,account=>$account,displayname=>$displayname,markname=>$markname);
+        }
+        if(defined $object){
+            if($object->set_markname($new_markname)){
+                $c->safe_render(json=>{code=>0,status=>"success"});
+            }
+            else{
+                $c->safe_render(json=>{code=>201,status=>"failure"});
+            }
+        }
+        else{$c->safe_render(json=>{code=>100,status=>"object not found"});}
 
     };
     any [qw(GET POST)] => '/openwx/make_friend' => sub{
@@ -511,11 +568,11 @@ sub call{
         my($id,$verify)= map {defined $_?Encode::encode("utf8",$_):$_} ($c->param("id"),$c->param("verify"));
         my $object;
         if($id eq $client->user->id){
-            $c->render(json=>{code=>101,status=>"can not be yourself"});
+            $c->safe_render(json=>{code=>101,status=>"can not be yourself"});
             return;
         }
         if(defined $client->search_friend(id=>$id)){
-            $c->render(json=>{code=>101,status=>"already a friend"});
+            $c->safe_render(json=>{code=>101,status=>"already a friend"});
             return;
         }
         for my $group ($client->groups){
@@ -523,15 +580,15 @@ sub call{
             last if defined $object;
         }
         if(not defined $object){
-            $c->render(json=>{code=>100,status=>"object not found"});
+            $c->safe_render(json=>{code=>100,status=>"object not found"});
             return;
         }
 
         if($object->make_friend($verify || '')){
-            $c->render(json=>{code=>0,status=>"success"});
+            $c->safe_render(json=>{code=>0,status=>"success"});
         }
         else{
-            $c->render(json=>{code=>201,status=>"failure"});
+            $c->safe_render(json=>{code=>201,status=>"failure"});
         }
     };
     any [qw(GET POST)] => '/openwx/get_avatar' => sub{
@@ -551,15 +608,15 @@ sub call{
                 $c->res->headers->expires(POSIX::strftime("%a, %d %b %Y %H:%M:%S GMT",gmtime($mtime+3600)));
                 $c->res->headers->last_modified(POSIX::strftime("%a, %d %b %Y %H:%M:%S GMT",gmtime($mtime)));
                 $c->res->headers->content_type($mime || 'image/jpg');
-                $c->render(data=>$data,);  
+                $c->safe_render(data=>$data,);  
             });
         }
-        else{$c->render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
+        else{$c->safe_render(json=>{msg_id=>undef,code=>100,status=>"object not found"});}
 
     };
     any [qw(GET POST)] => '/openwx/get_client_info' => sub{
         my $c = shift;
-        $c->render(json=>{
+        $c->safe_render(json=>{
             code=>0,
             pid=>$$,
             account=>$client->account,
@@ -576,7 +633,7 @@ sub call{
     };
     any [qw(GET POST)] => '/openwx/stop_client' => sub{
         my $c = shift;
-        $c->render(json=>{
+        $c->safe_render(json=>{
             code=>0,
             account=>$client->account,
             pid=>$$,
@@ -611,11 +668,11 @@ sub call{
                 media_ext   => $media_ext,
                 media_path  => $media_path,
             },
-            sub{my $json = shift;$client->reform_hash($json,1);$c->render(json=>$json) if defined $c}
+            sub{my $json = shift;$client->reform_hash($json,1);$c->safe_render(json=>$json) if defined $c}
         );
         
     };
-    any '/*whatever'  => sub{whatever=>'',$_[0]->render(text=>"api not found",status=>403)};
+    any '/*whatever'  => sub{whatever=>'',$_[0]->safe_render(text=>"api not found",status=>403)};
     package Mojo::Weixin::Plugin::Openwx;
     $server = Mojo::Weixin::Server->new();   
     $server->app($server->build_app("Mojo::Weixin::Plugin::Openwx::App"));
