@@ -26,6 +26,7 @@ sub call{
 
         $client->on(receive_message=>sub{
             my($client,$msg) = @_;
+            return if $msg !~ /^friend_message|group_message$/;
             return if not $msg->allow_plugin;
             return if not $onoff_flag;
             return if $is_need_at and $msg->type eq "group_message" and !$msg->is_at;
@@ -37,8 +38,8 @@ sub call{
             if($msg->sender->id eq $xiaoice->id){
                 my $binder = $db[0];
                 return if not defined $binder;
-                if($msg->format eq "media" and -e $msg->media_path){
-                    $binder->send_media($msg->media_path);
+                if($msg->format eq "media" and (defined $msg->media_id or  -e $msg->media_path) ){
+                    $binder->send_media({media_id=>$msg->media_id,media_path=>$msg->media_path});
                 }
                 else{
                     $binder->send($msg->content);
@@ -49,8 +50,8 @@ sub call{
                 if (@db == 0){
                     $db[0] = $object;
                     $msg->remove_at();
-                    if($msg->format eq "media" and -e $msg->media_path){
-                        $xiaoice->send_media($msg->media_path);
+                    if($msg->format eq "media" and (defined $msg->media_id or -e $msg->media_path) ){
+                        $xiaoice->send_media({media_id=>$msg->media_id,media_path=>$msg->media_path});
                     }
                     else{
                         $xiaoice->send($msg->content);
@@ -58,8 +59,8 @@ sub call{
                 }
                 elsif(@db > 0 and $db[0]->id eq $object->id){
                     $msg->remove_at();
-                    if($msg->format eq "media" and -e $msg->media_path){
-                        $xiaoice->send_media($msg->media_path);
+                    if($msg->format eq "media" and (defined $msg->media_id or -e $msg->media_path )){
+                        $xiaoice->send_media({media_id=>$msg->media_id,media_path=>$msg->media_path});
                     }
                     else{
                         $xiaoice->send($msg->content); 
